@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -33,26 +33,11 @@ namespace OrthancDatabases
     std::string  content_;
 
   public:
-    explicit BinaryStringValue()
-    {
-    }
-
     explicit BinaryStringValue(const std::string& content) :
       content_(content)
     {
     }
 
-    BinaryStringValue(const void* content,
-                      size_t size)
-    {
-      content_.assign(reinterpret_cast<const char*>(content), size);
-    }
-
-    std::string& GetContent()
-    {
-      return content_;
-    }
-
     const std::string& GetContent() const
     {
       return content_;
@@ -74,7 +59,5 @@ namespace OrthancDatabases
     }
     
     virtual IValue* Convert(ValueType target) const ORTHANC_OVERRIDE;
-
-    virtual std::string Format() const ORTHANC_OVERRIDE;
   };
 }
