@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -21,7 +21,9 @@
 
 #include "DatabaseManager.h"
 
-#include "../../Resources/Orthanc/Plugins/OrthancPluginCppWrapper.h"
+#include "Integer64Value.h"
+#include "BinaryStringValue.h"
+#include "Utf8StringValue.h"
 
 #include <Compatibility.h>  // For std::unique_ptr<>
 #include <Logging.h>
@@ -31,56 +33,6 @@
 
 namespace OrthancDatabases
 {
-  IDatabase& DatabaseManager::GetDatabase()
-  {
-    unsigned int maxConnectionRetries = 10;
-    unsigned int connectionRetryInterval = 5;
-    unsigned int count = 0;
-
-    factory_->GetConnectionRetriesParameters(maxConnectionRetries, connectionRetryInterval);
-      
-    while (database_.get() == NULL)
-    {
-      transaction_.reset(NULL);
-
-      try
-      {
-        database_.reset(factory_->Open());
-      }
-      catch (Orthanc::OrthancException& e)
-      {
-        if (e.GetErrorCode() == Orthanc::ErrorCode_DatabaseUnavailable)
-        {
-          count ++;
-
-          if (count <= maxConnectionRetries)
-          {
-            LOG(WARNING) << "Database is currently unavailable, retrying...";
-            boost::this_thread::sleep(boost::posix_time::seconds(connectionRetryInterval));
-            continue;
-          }
-          else
-          {
-            LOG(ERROR) << "Timeout when connecting to the database, giving up";
-          }
-        }
-
-        throw;
-      }
-    }
-
-    if (database_.get() == NULL ||
-        database_->GetDialect() != dialect_)
-    {
-      throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
-    }
-    else
-    {
-      return *database_;
-    }
-  }
-
-
   void DatabaseManager::Close()
   {
     LOG(TRACE) << "Closing the connection to the database";
@@ -108,7 +60,11 @@ namespace OrthancDatabases
     
   void DatabaseManager::CloseIfUnavailable(Orthanc::ErrorCode e)
   {
-    if (e != Orthanc::ErrorCode_Success)
+    if (e != Orthanc::ErrorCode_Success
+#if ORTHANC_FRAMEWORK_VERSION_IS_ABOVE(1, 9, 2)
+        && e != Orthanc::ErrorCode_DatabaseCannotSerialize
+#endif
+      )
     {
       transaction_.reset(NULL);
     }
@@ -165,7 +121,7 @@ namespace OrthancDatabases
 
       try
       {
-        transaction_.reset(GetDatabase().CreateTransaction(true));
+        transaction_.reset(GetDatabase().CreateTransaction(TransactionType_Implicit));
       }
       catch (Orthanc::OrthancException& e)
       {
@@ -200,22 +156,57 @@ namespace OrthancDatabases
   }
 
     
-  DatabaseManager::DatabaseManager(IDatabaseFactory* factory) :  // Takes ownership
-    factory_(factory)
+  DatabaseManager::DatabaseManager(IDatabaseFactory* factory) :
+    factory_(factory),
+    dialect_(Dialect_Unknown)
   {
     if (factory == NULL)
     {
       throw Orthanc::OrthancException(Orthanc::ErrorCode_NullPointer);
     }
-
-    dialect_ = factory->GetDialect();
   }
 
   
-  void DatabaseManager::StartTransaction()
+  IDatabase& DatabaseManager::GetDatabase()
   {
-    boost::recursive_mutex::scoped_lock lock(mutex_);
+    assert(factory_.get() != NULL);
+    
+    if (database_.get() == NULL)
+    {
+      database_.reset(factory_->Open());
 
+      if (database_.get() == NULL)
+      {
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
+      }
+
+      dialect_ = database_->GetDialect();
+      if (dialect_ == Dialect_Unknown)
+      {
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
+      }
+    }
+
+    return *database_;
+  }
+
+
+  Dialect DatabaseManager::GetDialect() const
+  {
+    if (database_.get() == NULL)
+    {
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
+    }
+    else
+    {
+      assert(dialect_ != Dialect_Unknown);
+      return dialect_;
+    }
+  }
+  
+
+  void DatabaseManager::StartTransaction(TransactionType type)
+  {
     try
     {
       if (transaction_.get() != NULL)
@@ -224,7 +215,7 @@ namespace OrthancDatabases
         throw Orthanc::OrthancException(Orthanc::ErrorCode_Database);
       }
 
-      transaction_.reset(GetDatabase().CreateTransaction(false));
+      transaction_.reset(GetDatabase().CreateTransaction(type));
     }
     catch (Orthanc::OrthancException& e)
     {
@@ -236,8 +227,6 @@ namespace OrthancDatabases
 
   void DatabaseManager::CommitTransaction()
   {
-    boost::recursive_mutex::scoped_lock lock(mutex_);
-
     if (transaction_.get() == NULL)
     {
       LOG(ERROR) << "Cannot commit a non-existing transaction";
@@ -261,11 +250,9 @@ namespace OrthancDatabases
 
   void DatabaseManager::RollbackTransaction()
   {
-    boost::recursive_mutex::scoped_lock lock(mutex_);
-
     if (transaction_.get() == NULL)
     {
-      LOG(ERROR) << "Cannot rollback a non-existing transaction";
+      LOG(INFO) << "Cannot rollback a non-existing transaction";
       throw Orthanc::OrthancException(Orthanc::ErrorCode_BadSequenceOfCalls);
     }
     else
@@ -284,19 +271,19 @@ namespace OrthancDatabases
   }
 
 
-  DatabaseManager::Transaction::Transaction(DatabaseManager& manager) :
-    lock_(manager.mutex_),
+  DatabaseManager::Transaction::Transaction(DatabaseManager& manager,
+                                            TransactionType type) :
     manager_(manager),
     database_(manager.GetDatabase()),
-    committed_(false)
+    active_(true)
   {
-    manager_.StartTransaction();
+    manager_.StartTransaction(type);
   }
 
 
   DatabaseManager::Transaction::~Transaction()
   {
-    if (!committed_)
+    if (active_)
     {
       try
       {
@@ -313,14 +300,28 @@ namespace OrthancDatabases
   
   void DatabaseManager::Transaction::Commit()
   {
-    if (committed_)
+    if (active_)
     {
-      throw Orthanc::OrthancException(Orthanc::ErrorCode_BadSequenceOfCalls);
+      manager_.CommitTransaction();
+      active_ = false;
     }
     else
     {
-      manager_.CommitTransaction();
-      committed_ = true;
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_BadSequenceOfCalls);
+    }
+  }
+
+
+  void DatabaseManager::Transaction::Rollback()
+  {
+    if (active_)
+    {
+      manager_.RollbackTransaction();
+      active_ = false;
+    }
+    else
+    {
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_BadSequenceOfCalls);
     }
   }
 
@@ -358,26 +359,19 @@ namespace OrthancDatabases
   
   void DatabaseManager::StatementBase::SetResult(IResult* result)
   {
-    std::unique_ptr<IResult> protection(result);
-    
-    if (result_.get() != NULL)
-    {
-      LOG(ERROR) << "Cannot execute twice a statement";
-      throw Orthanc::OrthancException(Orthanc::ErrorCode_BadSequenceOfCalls);
-    }
-
     if (result == NULL)
     {
       throw Orthanc::OrthancException(Orthanc::ErrorCode_NullPointer);
     }
-
-    result_.reset(protection.release());
+    else
+    {
+      result_.reset(result);
+    }
   }
 
   
   DatabaseManager::StatementBase::StatementBase(DatabaseManager& manager) :
     manager_(manager),
-    lock_(manager_.mutex_),
     transaction_(manager_.GetTransaction())
   {
   }
@@ -478,7 +472,72 @@ namespace OrthancDatabases
       manager_.CloseIfUnavailable(e.GetErrorCode());
       throw;
     }
-  }  
+  }
+
+
+  int64_t DatabaseManager::StatementBase::ReadInteger64(size_t field) const
+  {
+    if (IsDone())
+    {
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_Database);
+    }
+    else
+    {
+      const IValue& value = GetResultField(field);
+      
+      switch (value.GetType())
+      {
+        case ValueType_Integer64:
+          return dynamic_cast<const Integer64Value&>(value).GetValue();
+
+        default:
+          //LOG(ERROR) << value.Format();
+          throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
+      }
+    }
+  }
+
+
+  int32_t DatabaseManager::StatementBase::ReadInteger32(size_t field) const
+  {
+    if (IsDone())
+    {
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_Database);
+    }
+    else
+    {
+      int64_t value = ReadInteger64(field);
+
+      if (value != static_cast<int64_t>(static_cast<int32_t>(value)))
+      {
+        LOG(ERROR) << "Integer overflow";
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
+      }
+      else
+      {
+        return static_cast<int32_t>(value);
+      }
+    }
+  }
+
+    
+  std::string DatabaseManager::StatementBase::ReadString(size_t field) const
+  {
+    const IValue& value = GetResultField(field);
+
+    switch (value.GetType())
+    {
+      case ValueType_BinaryString:
+        return dynamic_cast<const BinaryStringValue&>(value).GetContent();
+
+      case ValueType_Utf8String:
+        return dynamic_cast<const Utf8StringValue&>(value).GetContent();
+
+      default:
+        //LOG(ERROR) << value.Format();
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
+    }
+  }
   
   
   DatabaseManager::CachedStatement::CachedStatement(const StatementLocation& location,
