@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2019 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -21,19 +21,24 @@
 
 #include "NullValue.h"
 
-#include <Core/OrthancException.h>
+#include "Utf8StringValue.h"
+
+#include <OrthancException.h>
 
 namespace OrthancDatabases
 {
   IValue* NullValue::Convert(ValueType target) const
   {
-    if (target == ValueType_Null)
+    switch (target)
     {
-      return new NullValue;
-    }
-    else
-    {
-      throw Orthanc::OrthancException(Orthanc::ErrorCode_ParameterOutOfRange);
+      case ValueType_Null:
+        return new NullValue;
+
+      case ValueType_Utf8String:
+        return new Utf8StringValue("(null)");
+
+      default:
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_ParameterOutOfRange);
     }
   }
 }
