@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -22,7 +22,6 @@
 #include "Utf8StringValue.h"
 
 #include "BinaryStringValue.h"
-#include "FileValue.h"
 #include "NullValue.h"
 #include "Integer64Value.h"
 
@@ -42,9 +41,6 @@ namespace OrthancDatabases
       case ValueType_BinaryString:
         return new BinaryStringValue(utf8_);
 
-      case ValueType_File:
-        return new FileValue(utf8_);
-
       case ValueType_Integer64:
         try
         {
@@ -58,6 +54,9 @@ namespace OrthancDatabases
 
         break;
 
+      case ValueType_Utf8String:
+        return new Utf8StringValue(utf8_);
+
       default:
         throw Orthanc::OrthancException(Orthanc::ErrorCode_ParameterOutOfRange);
     }
