@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -21,11 +21,12 @@
 
 #include "MySQLDatabase.h"
 
+#include "../Common/ImplicitTransaction.h"
+#include "../Common/Integer64Value.h"
+#include "../Common/RetryDatabaseFactory.h"
 #include "MySQLResult.h"
 #include "MySQLStatement.h"
 #include "MySQLTransaction.h"
-#include "../Common/ImplicitTransaction.h"
-#include "../Common/Integer64Value.h"
 
 #include <Compatibility.h>  // For std::unique_ptr<>
 #include <Logging.h>
@@ -51,6 +52,40 @@ namespace OrthancDatabases
   }
 
 
+  void MySQLDatabase::ThrowException()
+  {
+    LogError();
+
+    unsigned int error = mysql_errno(mysql_);
+    if (error == CR_SERVER_GONE_ERROR ||
+        error == CR_SERVER_LOST ||
+        error == ER_QUERY_INTERRUPTED)
+    {
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_DatabaseUnavailable);
+    }
+    else if (error == CR_COMMANDS_OUT_OF_SYNC)
+    {
+#if !defined(MARIADB_VERSION_ID)
+      LOG(ERROR) << "TODO - This error seems to be related to the use of libmysqlclient: Try to switch to mariadb-connector";
+#endif
+      
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_DatabaseUnavailable);
+    }
+    else if (error == ER_LOCK_DEADLOCK)
+    {
+#if ORTHANC_PLUGINS_VERSION_IS_ABOVE(1, 9, 2)
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_DatabaseCannotSerialize);
+#else
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_Database, "Collision between multiple writers");
+#endif
+    } 
+    else
+    {
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_Database);
+    }
+  }
+
+
   void MySQLDatabase::CheckErrorCode(int code)
   {
     if (code == 0)
@@ -59,19 +94,7 @@ namespace OrthancDatabases
     }
     else
     {
-      LogError();
-      
-      unsigned int error = mysql_errno(mysql_);
-      if (error == CR_SERVER_GONE_ERROR ||
-          error == CR_SERVER_LOST ||
-          error == ER_QUERY_INTERRUPTED)
-      {
-        throw Orthanc::OrthancException(Orthanc::ErrorCode_DatabaseUnavailable);
-      }
-      else
-      {
-        throw Orthanc::OrthancException(Orthanc::ErrorCode_Database);
-      }
+      ThrowException();
     }
   }
 
@@ -207,7 +230,7 @@ namespace OrthancDatabases
     const std::string& database = parameters.GetDatabase();
     
     {
-      MySQLTransaction t(db);
+      MySQLTransaction t(db, TransactionType_ReadWrite);
 
       if (!db.DoesDatabaseExist(t, database))
       {
@@ -215,8 +238,8 @@ namespace OrthancDatabases
         throw Orthanc::OrthancException(Orthanc::ErrorCode_UnknownResource);
       }
       
-      db.Execute("DROP DATABASE " + database, false);
-      db.Execute("CREATE DATABASE " + database, false);
+      db.ExecuteMultiLines("DROP DATABASE " + database, false);
+      db.ExecuteMultiLines("CREATE DATABASE " + database, false);
       t.Commit();
     }
   }
@@ -339,7 +362,7 @@ namespace OrthancDatabases
     {
       MySQLStatement statement(*this, query);
 
-      MySQLTransaction t(*this);
+      MySQLTransaction t(*this, TransactionType_ReadWrite);
       std::unique_ptr<IResult> result(t.Execute(statement, args));
 
       success = (!result->IsDone() &&
@@ -469,8 +492,8 @@ namespace OrthancDatabases
   }
 
 
-  void MySQLDatabase::Execute(const std::string& sql,
-                              bool arobaseSeparator)
+  void MySQLDatabase::ExecuteMultiLines(const std::string& sql,
+                                        bool arobaseSeparator)
   {
     if (mysql_ == NULL)
     {
@@ -519,44 +542,63 @@ namespace OrthancDatabases
     class MySQLImplicitTransaction : public ImplicitTransaction
     {
     private:
-      MySQLDatabase&  db_;
-
+      MySQLDatabase& db_;
+      
     protected:
       virtual IResult* ExecuteInternal(IPrecompiledStatement& statement,
-                                       const Dictionary& parameters)
+                                       const Dictionary& parameters) ORTHANC_OVERRIDE
       {
         return dynamic_cast<MySQLStatement&>(statement).Execute(*this, parameters);
       }
 
       virtual void ExecuteWithoutResultInternal(IPrecompiledStatement& statement,
-                                                const Dictionary& parameters)
+                                                const Dictionary& parameters) ORTHANC_OVERRIDE
       {
         dynamic_cast<MySQLStatement&>(statement).ExecuteWithoutResult(*this, parameters);
       }
-      
+
     public:
-      explicit MySQLImplicitTransaction(MySQLDatabase&  db) :
+      explicit MySQLImplicitTransaction(MySQLDatabase& db) :
         db_(db)
       {
       }
+      
+      virtual bool DoesTableExist(const std::string& name) ORTHANC_OVERRIDE
+      {
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError, "An explicit transaction is needed");
+      }
+
+      virtual bool DoesTriggerExist(const std::string& name) ORTHANC_OVERRIDE
+      {
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError, "An explicit transaction is needed");
+      }
+
+      virtual void ExecuteMultiLines(const std::string& query) ORTHANC_OVERRIDE
+      {
+        db_.ExecuteMultiLines(query, false /* don't deal with arobases */);
+      }
     };
   }
   
 
-  ITransaction* MySQLDatabase::CreateTransaction(bool isImplicit)
+  ITransaction* MySQLDatabase::CreateTransaction(TransactionType type)
   {
     if (mysql_ == NULL)
     {
       throw Orthanc::OrthancException(Orthanc::ErrorCode_BadSequenceOfCalls);
     }
 
-    if (isImplicit)
+    switch (type)
     {
-      return new MySQLImplicitTransaction(*this);
-    }
-    else
-    {
-      return new MySQLTransaction(*this);
+      case TransactionType_Implicit:
+        return new MySQLImplicitTransaction(*this);
+
+      case TransactionType_ReadOnly:
+      case TransactionType_ReadWrite:
+        return new MySQLTransaction(*this, type);
+
+      default:
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_ParameterOutOfRange);
     }
   }
 
@@ -617,4 +659,41 @@ namespace OrthancDatabases
   {
     database_.ReleaseAdvisoryLock(lock_);
   }
+
+  
+  class MySQLDatabase::Factory : public RetryDatabaseFactory
+  {
+  private:
+    MySQLParameters  parameters_;
+
+  protected:
+    virtual IDatabase* TryOpen()
+    {
+      std::unique_ptr<MySQLDatabase> db(new MySQLDatabase(parameters_));
+      db->Open();
+      db->ExecuteMultiLines("SET SESSION TRANSACTION ISOLATION LEVEL SERIALIZABLE", false);
+      return db.release();
+    }
+      
+  public:
+    explicit Factory(const MySQLParameters& parameters) :
+      RetryDatabaseFactory(parameters.GetMaxConnectionRetries(),
+                           parameters.GetConnectionRetryInterval()),
+      parameters_(parameters)
+    {
+    }
+  };
+
+
+  IDatabaseFactory* MySQLDatabase::CreateDatabaseFactory(const MySQLParameters& parameters)
+  {
+    return new Factory(parameters);
+  }
+
+
+  MySQLDatabase* MySQLDatabase::CreateDatabaseConnection(const MySQLParameters& parameters)
+  {
+    Factory factory(parameters);
+    return dynamic_cast<MySQLDatabase*>(factory.Open());
+  }
 }
