@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -25,7 +25,7 @@
 #  error MySQL support must be enabled to use this file
 #endif
 
-#include "../Common/IDatabase.h"
+#include "../Common/IDatabaseFactory.h"
 #include "MySQLParameters.h"
 
 #include <mysql.h>
@@ -37,6 +37,8 @@ namespace OrthancDatabases
   class MySQLDatabase : public IDatabase
   {
   private:
+    class Factory;
+    
     MySQLParameters  parameters_;
     MYSQL           *mysql_;
 
@@ -56,6 +58,8 @@ namespace OrthancDatabases
 
     void CheckErrorCode(int code);
 
+    void ThrowException();
+
     MYSQL* GetObject();
 
     void Open();
@@ -79,8 +83,8 @@ namespace OrthancDatabases
 
     void AdvisoryLock(const std::string& lock);
 
-    void Execute(const std::string& sql,
-                 bool arobaseSeparator);
+    void ExecuteMultiLines(const std::string& sql,
+                           bool arobaseSeparator);
 
     bool DoesTableExist(MySQLTransaction& transaction,
                         const std::string& name);
@@ -98,7 +102,7 @@ namespace OrthancDatabases
 
     virtual IPrecompiledStatement* Compile(const Query& query) ORTHANC_OVERRIDE;
 
-    virtual ITransaction* CreateTransaction(bool isImplicit) ORTHANC_OVERRIDE;
+    virtual ITransaction* CreateTransaction(TransactionType type) ORTHANC_OVERRIDE;
 
     static void GlobalFinalization();
 
@@ -116,5 +120,9 @@ namespace OrthancDatabases
 
       ~TransientAdvisoryLock();
     };
+
+    static IDatabaseFactory* CreateDatabaseFactory(const MySQLParameters& parameters);
+
+    static MySQLDatabase* CreateDatabaseConnection(const MySQLParameters& parameters);
   };
 }
