@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -43,17 +43,9 @@ namespace OrthancDatabases
         // SQL request is not a SELECT)
         done_ = true;
       }
-      else if (error == CR_SERVER_GONE_ERROR ||
-               error == CR_SERVER_LOST ||
-               error == ER_QUERY_INTERRUPTED)
-      {
-        database_.LogError();
-        throw Orthanc::OrthancException(Orthanc::ErrorCode_DatabaseUnavailable);
-      }
       else
       {
-        database_.LogError();
-        throw Orthanc::OrthancException(Orthanc::ErrorCode_Database);
+        database_.ThrowException();
       }
     }
     else
@@ -82,7 +74,7 @@ namespace OrthancDatabases
     // https://github.com/hholzgra/connector-c-examples/blob/master/mysql_stmt_bind_result.c
 
     SetFieldsCount(statement_.GetResultFieldsCount());
-
+    
     Step();
   }
     
