@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -22,7 +22,7 @@
 #include "MySQLStatement.h"
 
 #include "../Common/BinaryStringValue.h"
-#include "../Common/FileValue.h"
+#include "../Common/InputFileValue.h"
 #include "../Common/Integer64Value.h"
 #include "../Common/NullValue.h"
 #include "../Common/Utf8StringValue.h"
@@ -153,6 +153,7 @@ namespace OrthancDatabases
         case MYSQL_TYPE_STRING:
         case MYSQL_TYPE_VAR_STRING:
         case MYSQL_TYPE_BLOB:
+        case MYSQL_TYPE_LONG_BLOB:
           // https://medium.com/@adamhooper/in-mysql-never-use-utf8-use-utf8mb4-11761243e434
           switch (field.charsetnr)
           {
@@ -348,7 +349,6 @@ namespace OrthancDatabases
   MySQLStatement::MySQLStatement(MySQLDatabase& db,
                                  const Query& query) :
     db_(db),
-    readOnly_(query.IsReadOnly()),
     statement_(NULL),
     formatter_(Dialect_MySQL)
   {
@@ -500,9 +500,9 @@ namespace OrthancDatabases
           break;
         }
 
-        case ValueType_File:
+        case ValueType_InputFile:
         {
-          const std::string& content = dynamic_cast<const FileValue&>(value).GetContent();
+          const std::string& content = dynamic_cast<const InputFileValue&>(value).GetContent();
           inputs[i].buffer = const_cast<char*>(content.c_str());
           inputs[i].buffer_length = content.size();
           inputs[i].buffer_type = MYSQL_TYPE_BLOB;
