@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -39,7 +39,6 @@ namespace OrthancDatabases
     void Close();
 
     MySQLDatabase&             db_;
-    bool                       readOnly_;
     MYSQL_STMT                *statement_;
     GenericFormatter           formatter_;
     std::vector<ResultField*>  result_;
@@ -51,11 +50,6 @@ namespace OrthancDatabases
 
     virtual ~MySQLStatement();
 
-    virtual bool IsReadOnly() const ORTHANC_OVERRIDE
-    {
-      return readOnly_;
-    }
-
     MYSQL_STMT* GetObject();
 
     size_t GetResultFieldsCount() const
