@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -31,12 +31,25 @@
 
 namespace OrthancDatabases
 {
-  MySQLTransaction::MySQLTransaction(MySQLDatabase& db) :
+  MySQLTransaction::MySQLTransaction(MySQLDatabase& db,
+                                     TransactionType type) :
     db_(db),
-    readOnly_(true),
     active_(false)
   {
-    db_.Execute("START TRANSACTION", false);
+    switch (type)
+    {
+      case TransactionType_ReadWrite:
+        db_.ExecuteMultiLines("START TRANSACTION READ WRITE", false);
+        break;
+
+      case TransactionType_ReadOnly:
+        db_.ExecuteMultiLines("START TRANSACTION READ ONLY", false);
+        break;
+
+      default:
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_ParameterOutOfRange);
+    }
+        
     active_ = true;
   }
 
@@ -45,11 +58,11 @@ namespace OrthancDatabases
   {
     if (active_)
     {
-      LOG(WARNING) << "An active MySQL transaction was dismissed";
+      LOG(INFO) << "An active MySQL transaction was dismissed";
 
       try
       {
-        db_.Execute("ROLLBACK", false);
+        db_.ExecuteMultiLines("ROLLBACK", false);
       }
       catch (Orthanc::OrthancException&)
       {
@@ -63,9 +76,8 @@ namespace OrthancDatabases
   {
     if (active_)
     {
-      db_.Execute("ROLLBACK", false);
+      db_.ExecuteMultiLines("ROLLBACK", false);
       active_ = false;
-      readOnly_ = true;
     }
     else
     {
@@ -79,9 +91,8 @@ namespace OrthancDatabases
   {
     if (active_)
     {
-      db_.Execute("COMMIT", false);
+      db_.ExecuteMultiLines("COMMIT", false);
       active_ = false;
-      readOnly_ = true;
     }
     else
     {
@@ -94,14 +105,7 @@ namespace OrthancDatabases
   IResult* MySQLTransaction::Execute(IPrecompiledStatement& statement,
                                      const Dictionary& parameters)
   {
-    std::unique_ptr<IResult> result(dynamic_cast<MySQLStatement&>(statement).Execute(*this, parameters));
-
-    if (!statement.IsReadOnly())
-    {
-      readOnly_ = false;
-    }
-    
-    return result.release();
+    return dynamic_cast<MySQLStatement&>(statement).Execute(*this, parameters);
   }
 
 
@@ -109,10 +113,5 @@ namespace OrthancDatabases
                                               const Dictionary& parameters)
   {
     dynamic_cast<MySQLStatement&>(statement).ExecuteWithoutResult(*this, parameters);
-
-    if (!statement.IsReadOnly())
-    {
-      readOnly_ = false;
-    }
   }
 }
