@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -34,11 +34,11 @@ namespace OrthancDatabases
   {
   private:
     MySQLDatabase&  db_;
-    bool            readOnly_;
     bool            active_;
 
   public:
-    explicit MySQLTransaction(MySQLDatabase& db);
+    explicit MySQLTransaction(MySQLDatabase& db,
+                              TransactionType type);
 
     virtual ~MySQLTransaction();
 
@@ -47,11 +47,6 @@ namespace OrthancDatabases
       return false;
     }
     
-    virtual bool IsReadOnly() const ORTHANC_OVERRIDE
-    {
-      return readOnly_;
-    }
-
     virtual void Rollback() ORTHANC_OVERRIDE;
     
     virtual void Commit() ORTHANC_OVERRIDE;
@@ -61,5 +56,20 @@ namespace OrthancDatabases
 
     virtual void ExecuteWithoutResult(IPrecompiledStatement& transaction,
                                       const Dictionary& parameters) ORTHANC_OVERRIDE;
+
+    virtual bool DoesTableExist(const std::string& name) ORTHANC_OVERRIDE
+    {
+      return db_.DoesTableExist(*this, name);
+    }
+
+    virtual bool DoesTriggerExist(const std::string& name) ORTHANC_OVERRIDE
+    {
+      return db_.DoesTriggerExist(*this, name);
+    }
+
+    virtual void ExecuteMultiLines(const std::string& query) ORTHANC_OVERRIDE
+    {
+      db_.ExecuteMultiLines(query, false /* don't deal with arobases */);
+    }
   };
 }
