@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -23,13 +23,10 @@
 #include "../../Framework/MySQL/MySQLDatabase.h"
 #include "../../Framework/Plugins/PluginInitialization.h"
 
-#include <Compatibility.h>  // For std::unique_ptr<>
 #include <HttpClient.h>
 #include <Logging.h>
 #include <Toolbox.h>
 
-static std::unique_ptr<OrthancDatabases::MySQLIndex> backend_;
-
 
 extern "C"
 {
@@ -65,13 +62,12 @@ extern "C"
 
     try
     {
+      const size_t countConnections = mysql.GetUnsignedIntegerValue("IndexConnectionsCount", 1);
+
       OrthancDatabases::MySQLParameters parameters(mysql, configuration);
-
-      /* Create the database back-end */
-      backend_.reset(new OrthancDatabases::MySQLIndex(parameters));
-
-      /* Register the MySQL index into Orthanc */
-      OrthancPlugins::DatabaseBackendAdapter::Register(context, *backend_);
+      OrthancDatabases::IndexBackend::Register(
+        new OrthancDatabases::MySQLIndex(context, parameters), countConnections,
+        parameters.GetMaxConnectionRetries());
     }
     catch (Orthanc::OrthancException& e)
     {
@@ -91,8 +87,8 @@ extern "C"
   ORTHANC_PLUGINS_API void OrthancPluginFinalize()
   {
     LOG(WARNING) << "MySQL index is finalizing";
+    OrthancDatabases::IndexBackend::Finalize();
 
-    backend_.reset(NULL);
     OrthancDatabases::MySQLDatabase::GlobalFinalization();
     Orthanc::HttpClient::GlobalFinalize();
     Orthanc::Toolbox::FinalizeOpenSsl();
