@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -29,60 +29,37 @@ namespace OrthancDatabases
   class MySQLIndex : public IndexBackend 
   {
   private:
-    class Factory : public IDatabaseFactory
-    {
-    private:
-      MySQLIndex&  that_;
-
-    public:
-      Factory(MySQLIndex& that) :
-      that_(that)
-      {
-      }
-
-      virtual Dialect GetDialect() const
-      {
-        return Dialect_MySQL;
-      }
-
-      virtual IDatabase* Open()
-      {
-        return that_.OpenInternal();
-      }
-
-      virtual void GetConnectionRetriesParameters(unsigned int& maxConnectionRetries, unsigned int& connectionRetryInterval)
-      {
-        maxConnectionRetries = that_.parameters_.GetMaxConnectionRetries();
-        connectionRetryInterval = that_.parameters_.GetConnectionRetryInterval();
-      }
-    };
-
-    OrthancPluginContext*  context_;
     MySQLParameters        parameters_;
     bool                   clearAll_;
 
-    IDatabase* OpenInternal();
-
   public:
-    MySQLIndex(const MySQLParameters& parameters);
-
-    void SetOrthancPluginContext(OrthancPluginContext* context)
-    {
-      context_ = context;
-    }
+    MySQLIndex(OrthancPluginContext* context,
+               const MySQLParameters& parameters);
 
     void SetClearAll(bool clear)
     {
       clearAll_ = clear;
     }
 
-    virtual int64_t CreateResource(const char* publicId,
+    virtual IDatabaseFactory* CreateDatabaseFactory() ORTHANC_OVERRIDE;
+
+    virtual void ConfigureDatabase(DatabaseManager& database) ORTHANC_OVERRIDE;
+ 
+    virtual bool HasRevisionsSupport() const ORTHANC_OVERRIDE
+    {
+      return false;  // TODO - REVISIONS
+    }
+    
+    virtual int64_t CreateResource(DatabaseManager& manager,
+                                   const char* publicId,
                                    OrthancPluginResourceType type)
       ORTHANC_OVERRIDE;
 
-    virtual void DeleteResource(int64_t id) ORTHANC_OVERRIDE;
+    virtual void DeleteResource(IDatabaseBackendOutput& output,
+                                DatabaseManager& manager,
+                                int64_t id) ORTHANC_OVERRIDE;
 
-    virtual int64_t GetLastChangeIndex() ORTHANC_OVERRIDE;
+    virtual int64_t GetLastChangeIndex(DatabaseManager& manager) ORTHANC_OVERRIDE;
 
     virtual bool HasCreateInstance() const  ORTHANC_OVERRIDE
     {
@@ -91,6 +68,7 @@ namespace OrthancDatabases
 
 #if ORTHANC_PLUGINS_HAS_DATABASE_CONSTRAINT == 1
     virtual void CreateInstance(OrthancPluginCreateInstanceResult& result,
+                                DatabaseManager& manager,
                                 const char* hashPatient,
                                 const char* hashStudy,
                                 const char* hashSeries,
