@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -22,7 +22,7 @@
 #include "../Plugins/MySQLIndex.h"
 #include "../Plugins/MySQLStorageArea.h"
 
-OrthancDatabases::MySQLParameters globalParameters_;
+static OrthancDatabases::MySQLParameters globalParameters_;
 
 #include "../../Framework/Common/Integer64Value.h"
 #include "../../Framework/MySQL/MySQLDatabase.h"
@@ -47,20 +47,22 @@ TEST(MySQLIndex, Lock)
   OrthancDatabases::MySQLParameters lock = globalParameters_;
   lock.SetLock(true);
 
-  OrthancDatabases::MySQLIndex db1(noLock);
+  OrthancDatabases::MySQLIndex db1(NULL, noLock);
   db1.SetClearAll(true);
-  db1.Open();
+
+  std::unique_ptr<OrthancDatabases::DatabaseManager> manager1(OrthancDatabases::IndexBackend::CreateSingleDatabaseManager(db1));
 
   {
-    OrthancDatabases::MySQLIndex db2(lock);
-    db2.Open();
+    OrthancDatabases::MySQLIndex db2(NULL, lock);
+    std::unique_ptr<OrthancDatabases::DatabaseManager> manager2(OrthancDatabases::IndexBackend::CreateSingleDatabaseManager(db2));
+
+    OrthancDatabases::MySQLIndex db3(NULL, lock);
+    ASSERT_THROW(OrthancDatabases::IndexBackend::CreateSingleDatabaseManager(db3), Orthanc::OrthancException);
 
-    OrthancDatabases::MySQLIndex db3(lock);
-    ASSERT_THROW(db3.Open(), Orthanc::OrthancException);
   }
 
-  OrthancDatabases::MySQLIndex db4(lock);
-  db4.Open();
+  OrthancDatabases::MySQLIndex db4(NULL, lock);
+  std::unique_ptr<OrthancDatabases::DatabaseManager> manager4(OrthancDatabases::IndexBackend::CreateSingleDatabaseManager(db4));
 }
 
 
@@ -135,70 +137,146 @@ TEST(MySQL, DISABLED_Lock3)
 
 static int64_t CountFiles(OrthancDatabases::MySQLDatabase& db)
 {
-  OrthancDatabases::Query query("SELECT COUNT(*) FROM StorageArea", true);
-  OrthancDatabases::MySQLStatement s(db, query);
-  OrthancDatabases::MySQLTransaction t(db);
-  OrthancDatabases::Dictionary d;
-  std::unique_ptr<OrthancDatabases::IResult> result(s.Execute(t, d));
-  return dynamic_cast<const OrthancDatabases::Integer64Value&>(result->GetField(0)).GetValue();
+  OrthancDatabases::MySQLTransaction transaction(db, OrthancDatabases::TransactionType_ReadOnly);
+
+  int64_t count;
+  {
+    OrthancDatabases::Query query("SELECT COUNT(*) FROM StorageArea", true);
+    OrthancDatabases::MySQLStatement s(db, query);
+    OrthancDatabases::MySQLTransaction t(db, OrthancDatabases::TransactionType_ReadOnly);
+    OrthancDatabases::Dictionary d;
+    std::unique_ptr<OrthancDatabases::IResult> result(s.Execute(t, d));
+    count = dynamic_cast<const OrthancDatabases::Integer64Value&>(result->GetField(0)).GetValue();
+  }
+
+  transaction.Commit();
+  return count;
 }
 
 
 TEST(MySQL, StorageArea)
 {
-  OrthancDatabases::MySQLStorageArea storageArea(globalParameters_);
-  storageArea.SetClearAll(true);
+  std::unique_ptr<OrthancDatabases::MySQLDatabase> database(
+    OrthancDatabases::MySQLDatabase::CreateDatabaseConnection(globalParameters_));
+  
+  OrthancDatabases::MySQLStorageArea storageArea(globalParameters_, true /* clear database */);
 
   {
-    OrthancDatabases::DatabaseManager::Transaction transaction(storageArea.GetManager());
-    OrthancDatabases::MySQLDatabase& db = 
-      dynamic_cast<OrthancDatabases::MySQLDatabase&>(transaction.GetDatabase());
-
-    ASSERT_EQ(0, CountFiles(db));
+    std::unique_ptr<OrthancDatabases::StorageBackend::IAccessor> accessor(storageArea.CreateAccessor());
+    
+    ASSERT_EQ(0, CountFiles(*database));
   
     for (int i = 0; i < 10; i++)
     {
       std::string uuid = boost::lexical_cast<std::string>(i);
       std::string value = "Value " + boost::lexical_cast<std::string>(i * 2);
-      storageArea.Create(transaction, uuid, value.c_str(), value.size(), OrthancPluginContentType_Unknown);
+      accessor->Create(uuid, value.c_str(), value.size(), OrthancPluginContentType_Unknown);
     }
 
-    std::string tmp;
-    ASSERT_THROW(storageArea.ReadToString(tmp, transaction, "nope", OrthancPluginContentType_Unknown), 
+    std::string buffer;
+    ASSERT_THROW(OrthancDatabases::StorageBackend::ReadWholeToString(
+                   buffer, *accessor, "nope", OrthancPluginContentType_Unknown), 
                  Orthanc::OrthancException);
   
-    ASSERT_EQ(10, CountFiles(db));
-    storageArea.Remove(transaction, "5", OrthancPluginContentType_Unknown);
+    ASSERT_EQ(10, CountFiles(*database));
+    accessor->Remove("5", OrthancPluginContentType_Unknown);
 
-    ASSERT_EQ(9, CountFiles(db));
+    ASSERT_EQ(9, CountFiles(*database));
 
     for (int i = 0; i < 10; i++)
     {
       std::string uuid = boost::lexical_cast<std::string>(i);
       std::string expected = "Value " + boost::lexical_cast<std::string>(i * 2);
-      std::string content;
 
       if (i == 5)
       {
-        ASSERT_THROW(storageArea.ReadToString(content, transaction, uuid, OrthancPluginContentType_Unknown), 
+        ASSERT_THROW(OrthancDatabases::StorageBackend::ReadWholeToString(
+                       buffer, *accessor, uuid, OrthancPluginContentType_Unknown), 
                      Orthanc::OrthancException);
       }
       else
       {
-        storageArea.ReadToString(content, transaction, uuid, OrthancPluginContentType_Unknown);
-        ASSERT_EQ(expected, content);
+        OrthancDatabases::StorageBackend::ReadWholeToString(buffer, *accessor, uuid, OrthancPluginContentType_Unknown);
+        ASSERT_EQ(expected, buffer);
       }
     }
 
     for (int i = 0; i < 10; i++)
     {
-      storageArea.Remove(transaction, boost::lexical_cast<std::string>(i),
-                         OrthancPluginContentType_Unknown);
+      accessor->Remove(boost::lexical_cast<std::string>(i), OrthancPluginContentType_Unknown);
     }
 
-    ASSERT_EQ(0, CountFiles(db));
+    ASSERT_EQ(0, CountFiles(*database));
+  }
+}
 
-    transaction.Commit();
+
+TEST(MySQL, StorageReadRange)
+{
+  std::unique_ptr<OrthancDatabases::MySQLDatabase> database(
+    OrthancDatabases::MySQLDatabase::CreateDatabaseConnection(globalParameters_));
+  
+  OrthancDatabases::MySQLStorageArea storageArea(globalParameters_, true /* clear database */);
+
+  {
+    std::unique_ptr<OrthancDatabases::StorageBackend::IAccessor> accessor(storageArea.CreateAccessor());
+    ASSERT_EQ(0, CountFiles(*database));  
+    accessor->Create("uuid", "abcd\0\1\2\3\4\5", 10, OrthancPluginContentType_Unknown);
+    ASSERT_EQ(1u, CountFiles(*database));  
+  }
+
+  {
+    std::unique_ptr<OrthancDatabases::StorageBackend::IAccessor> accessor(storageArea.CreateAccessor());
+    ASSERT_EQ(1u, CountFiles(*database));
+
+    std::string s;
+    OrthancDatabases::StorageBackend::ReadWholeToString(s, *accessor, "uuid", OrthancPluginContentType_Unknown);
+    ASSERT_EQ(10u, s.size());
+    ASSERT_EQ('a', s[0]);
+    ASSERT_EQ('d', s[3]);
+    ASSERT_EQ('\0', s[4]);
+    ASSERT_EQ('\5', s[9]);
+
+    OrthancDatabases::StorageBackend::ReadRangeToString(s, *accessor, "uuid", OrthancPluginContentType_Unknown, 0, 0);
+    ASSERT_TRUE(s.empty());
+
+    OrthancDatabases::StorageBackend::ReadRangeToString(s, *accessor, "uuid", OrthancPluginContentType_Unknown, 0, 1);
+    ASSERT_EQ(1u, s.size());
+    ASSERT_EQ('a', s[0]);
+
+    OrthancDatabases::StorageBackend::ReadRangeToString(s, *accessor, "uuid", OrthancPluginContentType_Unknown, 4, 1);
+    ASSERT_EQ(1u, s.size());
+    ASSERT_EQ('\0', s[0]);
+
+    OrthancDatabases::StorageBackend::ReadRangeToString(s, *accessor, "uuid", OrthancPluginContentType_Unknown, 9, 1);
+    ASSERT_EQ(1u, s.size());
+    ASSERT_EQ('\5', s[0]);
+
+    OrthancDatabases::StorageBackend::ReadRangeToString(s, *accessor, "uuid", OrthancPluginContentType_Unknown, 10, 0);
+    ASSERT_TRUE(s.empty());
+
+    // Cannot read non-empty range after the end of the string
+    ASSERT_THROW(OrthancDatabases::StorageBackend::ReadRangeToString(
+                   s, *accessor, "uuid", OrthancPluginContentType_Unknown, 10, 1), Orthanc::OrthancException);
+
+    OrthancDatabases::StorageBackend::ReadRangeToString(s, *accessor, "uuid", OrthancPluginContentType_Unknown, 0, 4);
+    ASSERT_EQ(4u, s.size());
+    ASSERT_EQ('a', s[0]);
+    ASSERT_EQ('b', s[1]);
+    ASSERT_EQ('c', s[2]);
+    ASSERT_EQ('d', s[3]);
+
+    OrthancDatabases::StorageBackend::ReadRangeToString(s, *accessor, "uuid", OrthancPluginContentType_Unknown, 4, 6);
+    ASSERT_EQ(6u, s.size());
+    ASSERT_EQ('\0', s[0]);
+    ASSERT_EQ('\1', s[1]);
+    ASSERT_EQ('\2', s[2]);
+    ASSERT_EQ('\3', s[3]);
+    ASSERT_EQ('\4', s[4]);
+    ASSERT_EQ('\5', s[5]);
+
+    ASSERT_THROW(OrthancDatabases::StorageBackend::ReadRangeToString(
+                   s, *accessor, "uuid", OrthancPluginContentType_Unknown, 4, 7), Orthanc::OrthancException);
   }
 }
 
@@ -210,13 +288,13 @@ TEST(MySQL, ImplicitTransaction)
   db.Open();
 
   {
-    OrthancDatabases::MySQLTransaction t(db);
+    OrthancDatabases::MySQLTransaction t(db, OrthancDatabases::TransactionType_ReadOnly);
     ASSERT_FALSE(db.DoesTableExist(t, "test"));
     ASSERT_FALSE(db.DoesTableExist(t, "test2"));
   }
 
   {
-    std::unique_ptr<OrthancDatabases::ITransaction> t(db.CreateTransaction(false));
+    std::unique_ptr<OrthancDatabases::ITransaction> t(db.CreateTransaction(OrthancDatabases::TransactionType_ReadWrite));
     ASSERT_FALSE(t->IsImplicit());
   }
 
@@ -224,7 +302,7 @@ TEST(MySQL, ImplicitTransaction)
     OrthancDatabases::Query query("CREATE TABLE test(id INT)", false);
     std::unique_ptr<OrthancDatabases::IPrecompiledStatement> s(db.Compile(query));
     
-    std::unique_ptr<OrthancDatabases::ITransaction> t(db.CreateTransaction(true));
+    std::unique_ptr<OrthancDatabases::ITransaction> t(db.CreateTransaction(OrthancDatabases::TransactionType_Implicit));
     ASSERT_TRUE(t->IsImplicit());
     ASSERT_THROW(t->Commit(), Orthanc::OrthancException);
     ASSERT_THROW(t->Rollback(), Orthanc::OrthancException);
@@ -242,14 +320,14 @@ TEST(MySQL, ImplicitTransaction)
     OrthancDatabases::Query query("CREATE TABLE test2(id INT)", false);
     std::unique_ptr<OrthancDatabases::IPrecompiledStatement> s(db.Compile(query));
     
-    std::unique_ptr<OrthancDatabases::ITransaction> t(db.CreateTransaction(true));
+    std::unique_ptr<OrthancDatabases::ITransaction> t(db.CreateTransaction(OrthancDatabases::TransactionType_Implicit));
 
     OrthancDatabases::Dictionary args;
     t->ExecuteWithoutResult(*s, args);
   }
 
   {
-    OrthancDatabases::MySQLTransaction t(db);
+    OrthancDatabases::MySQLTransaction t(db, OrthancDatabases::TransactionType_ReadOnly);
     ASSERT_TRUE(db.DoesTableExist(t, "test"));
     ASSERT_TRUE(db.DoesTableExist(t, "test2"));
   }
